$fn = 45;

mount_screw_dia = 11/64.0 * 25.4;
mount_screw_hole_radius = mount_screw_dia * 1.05 / 2.0;

fan_screw_hole_size = 10/64.0 * 25.4 / 2.0 * 1.085;
fan_dia = 80;
fan_hole_spacing = 71.6;
fan_buffer = (fan_dia - fan_hole_spacing - 4.5)/2;

fan_margin = 3.8;
mount_thickness = 2.5;
wall_thickness = 1.9;

plate_width = fan_margin * 2 + fan_dia ;

target_fan_width = 60;

adapter_distance = target_fan_width * 0.255;

taper_ellipse_ratio = adapter_distance / ((plate_width - target_fan_width)/2);
fan_thickness = 9.5;
fan_gap = fan_thickness + 1.33;

screwdriver_hole_radius = 11/32.0 * 25.4 / 2.0;

slope_angle = atan((fan_margin * 2 + fan_dia  - target_fan_width )/(adapter_distance));

module taper_ellipse(){
 scale([1,1,taper_ellipse_ratio]){
    rotate([0,90,0]){      
      cylinder(h = wall_thickness*2, r = (plate_width - target_fan_width)/2);
    }
  } 
}

module taper_wall(x_factor, y_factor, q_rot){
  rotate([0,90,0]){      
    difference(){    
      linear_extrude(height = fan_margin * 2 + fan_dia + wall_thickness){
        difference(){
          scale([taper_ellipse_ratio,1]){
            circle(r = (plate_width - target_fan_width)/2); 
          }
          
          translate([x_factor * wall_thickness, y_factor * wall_thickness]){
            scale([taper_ellipse_ratio,1]){
              circle(r = (plate_width - target_fan_width)/2); 
            }
          }                
        }
      }
      translate([0,-adapter_distance * 2, -2 * wall_thickness]){
        cube([adapter_distance,adapter_distance*4,fan_margin * 2 + fan_dia + wall_thickness * 4]);
      }
      
    translate([0, 0,-1]){
        rotate([0,0,q_rot]) {
          cube([2 *(plate_width - target_fan_width) * 2,wall_thickness + 2 * plate_width - target_fan_width + 10,fan_margin * 2 + fan_dia + wall_thickness * 4]);
        }
      }
    }
  }
}


union(){
  
   translate([fan_margin * 2 + fan_dia,-1 * wall_thickness, -1 * (fan_gap + wall_thickness) -1.1]){
    difference(){   
      cube([wall_thickness, plate_width + 2*wall_thickness ,  (fan_gap + wall_thickness) + mount_thickness + 1.1]);
    
  
      translate([-1,wall_thickness, fan_gap /-2.0]){
        rotate([0,90,0]){                
          union(){      
            translate([-1 * (fan_gap + wall_thickness),(plate_width/6.0),0]){
              cylinder(r= screwdriver_hole_radius, h = wall_thickness * 2);
            }
            
            translate([-1 * (fan_gap + wall_thickness),(plate_width * 5/6.0),0]){
              cylinder(r= screwdriver_hole_radius, h = wall_thickness * 2);
            }
            translate([-1 * (fan_gap + wall_thickness),(plate_width /2.0),0]){
              cylinder(r= screwdriver_hole_radius, h = wall_thickness * 2);
            }
          }         
      }
    }
  }
}
  
 translate([wall_thickness * -1, -1 *wall_thickness,-1 * (fan_gap + wall_thickness)]){
  difference(){
    cube([wall_thickness,plate_width + 2*wall_thickness,fan_gap + wall_thickness + mount_thickness]);
    translate([-1,wall_thickness, fan_gap /-2.0]){
    rotate([0,90,0]){      
      
      union(){      
        translate([-1 * (fan_gap + wall_thickness),(plate_width/6.0),0]){
          cylinder(r= mount_screw_hole_radius, h = wall_thickness * 2);
        }
        
        translate([-1 * (fan_gap + wall_thickness),(plate_width * 5/6.0),0]){
          cylinder(r= mount_screw_hole_radius, h = wall_thickness * 2);
        }
        translate([-1 * (fan_gap + wall_thickness),(plate_width /2.0),0]){
          cylinder(r= mount_screw_hole_radius, h = wall_thickness * 2);
        }
      }
      
      
  }
}
  }
 } 
 
 rotate([90,0,0]){
   linear_extrude(height = wall_thickness){
    polygon(points = [[0,-1 * (fan_gap + wall_thickness)], [fan_margin * 2 + fan_dia, -1 * (fan_gap + wall_thickness)], [fan_margin * 2 + fan_dia, mount_thickness], [0,mount_thickness]]);
   }      
 }
 
 translate([0, fan_dia + 2*fan_margin + wall_thickness,0]){
 rotate([90,0,0]){
   linear_extrude(height = wall_thickness){
    polygon(points = [[0,-1 * (fan_gap + wall_thickness)], [fan_margin * 2 + fan_dia, -1 * (fan_gap + wall_thickness)], [fan_margin * 2 + fan_dia, mount_thickness], [0,mount_thickness]]);
   }      
 }
 }
 
difference(){
 color("blue") cube([fan_margin * 2 + fan_dia, fan_margin * 2 + fan_dia, mount_thickness]);
  union(){
    
      // fan hole 
    translate([fan_margin,fan_margin,-1]){
      translate([(fan_dia+0.5)/2,(fan_dia+0/5)/2,0]){
        cylinder(r=(fan_dia+0.5)/2, h = mount_thickness * 2);
      }
      
      translate([fan_screw_hole_size + fan_buffer,fan_screw_hole_size + fan_buffer,0]){
        cylinder(r = fan_screw_hole_size, h = mount_thickness * 2);
      }        
      translate([fan_screw_hole_size + fan_buffer + fan_hole_spacing,fan_screw_hole_size + fan_buffer,0]){
        cylinder(r = fan_screw_hole_size, h = mount_thickness * 2);
      }        
      translate([fan_screw_hole_size + fan_buffer + fan_hole_spacing,fan_screw_hole_size + fan_buffer + fan_hole_spacing,0]){
        cylinder(r = fan_screw_hole_size, h = mount_thickness * 2);
      }        
      translate([fan_screw_hole_size + fan_buffer,fan_screw_hole_size + fan_buffer + fan_hole_spacing,0]){
        cylinder(r = fan_screw_hole_size, h = mount_thickness * 2);
      }        

    }
  }
}

translate([0,0,-1 * fan_gap]){
  difference(){
  translate([-1 * wall_thickness, 0,-1 * adapter_distance]){
    cube([wall_thickness, plate_width, adapter_distance]);
  }

  translate([-1.1 * wall_thickness, 0, -1 * adapter_distance]){
      taper_ellipse();
  }

  translate([-1.1 * wall_thickness, plate_width, -1 * adapter_distance]){
      taper_ellipse();
  }
  }

  union(){
    difference(){
      union(){
        
        translate([-1 * wall_thickness, 0, -1 * adapter_distance]){
          color("red") taper_wall(1, -1, 180); // TODO unfuck me
        }
        
        translate([-1 * wall_thickness, plate_width, -1 * adapter_distance]){
          taper_wall(1, 1, 90);
        }
      }
        
      translate([target_fan_width, 0, adapter_distance * -1 - 1]){
        rotate([0,slope_angle,0]){  
          cube([ fan_margin * 2 + fan_dia  - target_fan_width,plate_width , adapter_distance  * 2]);
        }
      }  
    }
    
    
    
    difference(){
      translate([target_fan_width, 0, adapter_distance * -1 - 1]){
        rotate([0,slope_angle,0]){  
          cube([ wall_thickness,plate_width , sqrt( pow(adapter_distance,2.0) + pow((fan_margin * 2 + fan_dia  - target_fan_width) ,2))]);
        }
      }
    
    translate([0,0,-1 * adapter_distance]){
      rotate([0,90,0]){
        linear_extrude(height = fan_margin * 2 + fan_dia + wall_thickness){                 
            translate([1 * wall_thickness, -1 * wall_thickness]){
              scale([taper_ellipse_ratio,1]){
                circle(r = (plate_width - target_fan_width)/2); 
              }
            }                        
        }
      }
      
      translate([0,plate_width,0]){
          rotate([0,90,0]){
        linear_extrude(height = fan_margin * 2 + fan_dia + wall_thickness){                 
            translate([1 * wall_thickness, 1 * wall_thickness]){
              scale([taper_ellipse_ratio,1]){
                circle(r = (plate_width - target_fan_width)/2); 
              }
            }                        
        }
      }
      }
      
    }
      
     
    
    translate([0,0,-1 * adapter_distance - 10]){
      cube([target_fan_width  * 2,plate_width,10]);
    }
    
    
    translate([0,-1,-1 * wall_thickness]){
      cube([fan_margin * 2 + fan_dia + wall_thickness, plate_width * 2, 10]);
    }
    }

  }
  
}


}


